@@ -19,14 +19,14 @@
 
 import sqlite3
 import os
-
+from typing import Any, Dict, Iterable, List, Protocol, Union, cast
 
 class DbManager(object):
     """
     Class to manage an SQLite database.
     """
 
-    def __init__(self, root_path, db_name, db_version):
+    def __init__(self, root_path: str, db_name: str, db_version: int):
         self.root_path = root_path
         self.db_name = db_name
         self.db_version = db_version
@@ -94,6 +94,17 @@ class DbManager(object):
         cursor.executemany(insert_many_command, data)
         self.connection.commit()
 
+    def delete_where(self, table_name: str, where: list[str]):
+        """
+        Deletes the rows of the given table that matches the given where conditions
+        :param table_name: str
+        :param where: [str]
+        """
+        cursor = self.connection.cursor()
+        delete_command = self.generate_delete_command(table_name, where[:])
+        cursor.execute(delete_command)
+        self.connection.commit()
+
     def select_first(self, table_name):
         """
         Returns the first row of the given table
@@ -122,16 +133,6 @@ class DbManager(object):
         cursor = self._select_with_all_fields(table_name)
         return cursor.fetchall()
 
-    def select_all_where(self, table_name, where):
-        """
-        Returns all the rows of the given table that matches the given where conditions
-        :param table_name: str
-        :param where: [str]
-        :return rows: [[]]
-        """
-        cursor = self._select_with_all_fields(table_name, where)
-        return cursor.fetchall()
-
     def count(self, table_name):
         """
         Returns the number of rows of the given table
@@ -152,7 +153,7 @@ class DbManager(object):
         if os.path.exists(self._get_db_filepath()):
             os.remove(self._get_db_filepath())
 
-    def _get_db_filepath(self):
+    def _get_db_filepath(self) -> str:
         """
         Returns the path of the .db file
         :return path: int
@@ -172,7 +173,7 @@ class DbManager(object):
         self.insert(options_table_name, columns, ['name', self.db_name])
         self.insert(options_table_name, columns, ['version', self.db_version])
 
-    def _select_with_all_fields(self, table_name, where=[]):
+    def _select_with_all_fields(self, table_name: str, where: List[str] = []) -> sqlite3.Cursor:
         """
         Returns the cursor of the select command with the given parameters
         :param table_name: str
@@ -189,7 +190,7 @@ class DbManager(object):
     """
 
     @staticmethod
-    def generate_create_table_command(table_name, fields):
+    def generate_create_table_command(table_name: str, fields: List[str]) -> str:
         create_command = 'CREATE TABLE IF NOT EXISTS ' + table_name + ' (' + fields.pop(0)
         for field in fields:
             create_command += (', ' + field)
@@ -197,12 +198,14 @@ class DbManager(object):
         return create_command
 
     @staticmethod
-    def generate_drop_table_command(table_name):
+    def generate_drop_table_command(table_name: str) -> str:
         drop_command = 'DROP TABLE IF EXISTS ' + table_name
         return drop_command
 
     @staticmethod
-    def generate_insert_command(table_name, columns, values):
+    def generate_insert_command(
+        table_name: str, columns: List[str], values: List[str]
+    ) -> str:
         insert_command = 'INSERT INTO ' + table_name + '(' + columns.pop(0)
         for column in columns:
             insert_command += (', ' + column)
@@ -213,7 +216,7 @@ class DbManager(object):
         return insert_command
 
     @staticmethod
-    def generate_insert_many_command(table_name, num_of_values):
+    def generate_insert_many_command(table_name: str, num_of_values: int) -> str:
         insert_command = 'INSERT INTO ' + table_name + ' VALUES (?'
         num_of_values -= 1
         while num_of_values > 0:
@@ -223,14 +226,78 @@ class DbManager(object):
         return insert_command
 
     @staticmethod
-    def generate_count_command(table_name):
+    def generate_count_command(table_name: str) -> str:
         count_command = 'SELECT count(*) FROM ' + table_name
         return count_command
 
     @staticmethod
-    def generate_select_command(table_name, where=[]):
+    def generate_select_command(table_name: str, where: List[str] = []) -> str:
         basic_select = 'SELECT * FROM ' + table_name
         select_command = basic_select if len(where) == 0 else basic_select + ' WHERE ' + where.pop(0)
         for condition in where:
             select_command += ' AND ' + condition
         return select_command
+    
+    @staticmethod
+    def generate_delete_command(table_name: str, where: List[str] = []) -> str:
+        delete_command = "DELETE FROM " + table_name + " WHERE " + where.pop(0)
+        for condition in where:
+            delete_command += " AND " + condition
+        return delete_command
+
+class DatabaseManager(Protocol):
+    """Common interface for database managers.
+    We used a protocol here to avoid having to modify the existing
+    SQLite code (as we would if we used an abstract/ABC class).
+    And the new database manager will "quack" like the other one does.
+    """
+
+    connection: Union[sqlite3.Connection]
+
+    def backup(self): ...
+    def restore(self): ...
+    def disconnect(self): ...
+    def create_table(self, table_name: str, fields: List[str]): ...
+    def drop_table(self, table_name: str): ...
+    def insert(self, table_name: str, columns: List[str], values: List[str]): ...
+    def insertMany(self, table_name: str, data: List[Union[Iterable, Dict]]): ...
+    def delete_where(self, table_name: str, where: List[str]): ...
+    def select_first(self, table_name: str) -> List[Any]: ...
+    def select_first_where(self, table_name: str, where: List[str]) -> List[Any]: ...
+    def select_all(self, table_name: str) -> List[List[Any]]: ...
+    def select_all_where(self, table_name: str, where: List[str]) -> List[List[Any]]: ...
+    def count(self, table_name: str) -> int: ...
+    def drop(self): ...
+
+
+def create_db_manager(db_engine: str, **options) -> DatabaseManager:
+    """
+    Creates a Postgres or SQLite database manager based on the Autosubmit configuration.
+    Note that you must provide the options even if they are optional, in which case
+    you must provide ``options=None``, or you will get a ``KeyError``.
+    Later we might be able to drop the SQLite database manager. So, for the moment,
+    please call the function providing the database engine type, and the arguments
+    for both SQLite and for SQLAlchemy.
+    This means you do not have to do an ``if/else`` in your code, just give
+    this function the engine type, and all the valid options, and it should
+    handle choosing and building the database manager for you. e.g.
+    ```python
+    from autosubmit.database.db_manager import create_db_manager
+    options = {
+        # these are for sqlite
+        'root_path': '/tmp/',
+        'db_name': 'name.db',
+        'db_version': 1,
+        # and these for sqlalchemy -- not very elegant, but this is
+        # to work-effectively-with-legacy-code (as in that famous book).
+        'schema': 'a001'
+    }
+    db_manager = create_db_manager(db_engine='postgres', **options)
+    ```
+    :param db_engine: The database engine type.
+    :return: A ``DatabaseManager``.
+    :raises ValueError: If the database engine type is not valid.
+    :raises KeyError: If the ``options`` dictionary is missing a required parameter for an engine.
+    """
+    # TODO Create SqlAlchemyDbManager and add it to the if/else
+    return cast(DatabaseManager, DbManager(options['root_path'], options['db_name'], options['db_version']))
\ No newline at end of file
