@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-
+import gc
 # Copyright 2017-2020 Earth Sciences Department, BSC-CNS
 
 # This file is part of Autosubmit.
@@ -13,16 +13,18 @@
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
 # You should have received a copy of the GNU General Public License
 # along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.
-import pickle
-from sys import setrecursionlimit
 
 import os
-
-from log.log import Log
-from autosubmit.database.db_manager import DbManager
+import pickle
+from sys import setrecursionlimit, getrecursionlimit
+import shutil
+from autosubmit.database.db_manager import create_db_manager
+from log.log import AutosubmitCritical, Log
+from contextlib import suppress
+from autosubmitconfigparser.config.basicconfig import BasicConfig
+from pathlib import Path
 
 
 class JobListPersistence(object):
@@ -31,7 +33,7 @@ class JobListPersistence(object):
 
     """
 
-    def save(self, persistence_path, persistence_file, job_list):
+    def save(self, persistence_path, persistence_file, job_list , graph):
         """
         Persists a job list
         :param job_list: JobList
@@ -67,14 +69,31 @@ class JobListPersistencePkl(JobListPersistence):
 
         """
         path = os.path.join(persistence_path, persistence_file + '.pkl')
-        if os.path.exists(path):
-            fd = open(path, 'rb')
-            return pickle.load(fd)
-        else:
-            Log.printlog('File {0} does not exist'.format(path),Log.WARNING)
-            return list()
+        path_tmp = os.path.join(persistence_path[:-3]+"tmp", persistence_file + f'.pkl.tmp_{os.urandom(8).hex()}')
 
-    def save(self, persistence_path, persistence_file, job_list):
+        try:
+            open(path).close()
+        except PermissionError:
+            Log.warning(f'Permission denied to read {path}')
+            raise
+        except FileNotFoundError:
+            Log.warning(f'File {path} does not exist. ')
+            raise
+        else:
+            # copy the path to a tmp file randomseed to avoid corruption
+            try:
+                shutil.copy(path, path_tmp)
+                with open(path_tmp, 'rb') as fd:
+                    current_limit = getrecursionlimit()
+                    setrecursionlimit(100000)
+                    job_list = pickle.load(fd)
+                    setrecursionlimit(current_limit)
+            finally:
+                os.remove(path_tmp)
+
+            return job_list
+
+    def save(self, persistence_path, persistence_file, job_list, graph):
         """
         Persists a job list in a pkl file
         :param job_list: JobList
@@ -82,17 +101,20 @@ class JobListPersistencePkl(JobListPersistence):
         :param persistence_path: str
 
         """
-        path = os.path.join(persistence_path, persistence_file + '.pkl')
-        fd = open(path, 'wb')
-        setrecursionlimit(50000)
+
+        path = os.path.join(persistence_path, persistence_file + '.pkl' + '.tmp')
+        with suppress(FileNotFoundError, PermissionError):
+            os.remove(path)
         Log.debug("Saving JobList: " + path)
-        jobs_data = [(job.name, job.id, job.status,
-                      job.priority, job.section, job.date,
-                      job.member, job.chunk,
-                      job.local_logs[0], job.local_logs[1],
-                      job.remote_logs[0], job.remote_logs[1],job.wrapper_type) for job in job_list]
-        pickle.dump(jobs_data, fd, protocol=2)
-        Log.debug('Job list saved')
+        with open(path, 'wb') as fd:
+            current_limit = getrecursionlimit()
+            setrecursionlimit(100000)
+            pickle.dump({job.name: job.__getstate__() for job in job_list}, fd, pickle.HIGHEST_PROTOCOL)
+            setrecursionlimit(current_limit)
+            gc.collect()  # Tracemalloc show leaks without this
+        os.replace(path, path[:-4])
+        Log.debug(f'JobList saved in {path[:-4]}')
+
 
 
 class JobListPersistenceDb(JobListPersistence):
@@ -103,13 +125,32 @@ class JobListPersistenceDb(JobListPersistence):
 
     VERSION = 3
     JOB_LIST_TABLE = 'job_list'
-    TABLE_FIELDS = ['name', 'id', 'status', 'priority',
-                    'section', 'date', 'member', 'chunk',
-                    'local_out', 'local_err',
-                    'remote_out', 'remote_err']
+    TABLE_FIELDS = [
+        "name",
+        "id",
+        "status",
+        "priority",
+        "section",
+        "date",
+        "member",
+        "chunk",
+        "split",
+        "local_out",
+        "local_err",
+        "remote_out",
+        "remote_err",
+        "wrapper_type",
+    ]
 
-    def __init__(self, persistence_path, persistence_file):
-        self.db_manager = DbManager(persistence_path, persistence_file, self.VERSION)
+    def __init__(self, expid: str):
+        options = {
+            "root_path": str(Path(BasicConfig.LOCAL_ROOT_DIR, expid, "pkl")),
+            "db_name": f"job_list_{expid}",
+            "db_version": self.VERSION,
+            "schema": expid
+        }
+        self.expid = expid
+        self.db_manager = create_db_manager(BasicConfig.DATABASE_BACKEND, **options)
 
     def load(self, persistence_path, persistence_file):
         """
@@ -120,7 +161,7 @@ class JobListPersistenceDb(JobListPersistence):
         """
         return self.db_manager.select_all(self.JOB_LIST_TABLE)
 
-    def save(self, persistence_path, persistence_file, job_list):
+    def save(self, persistence_path, persistence_file, job_list, graph):
         """
         Persists a job list in a database
         :param job_list: JobList
@@ -131,7 +172,7 @@ class JobListPersistenceDb(JobListPersistence):
         self._reset_table()
         jobs_data = [(job.name, job.id, job.status,
                       job.priority, job.section, job.date,
-                      job.member, job.chunk,
+                      job.member, job.chunk, job.split,
                       job.local_logs[0], job.local_logs[1],
                       job.remote_logs[0], job.remote_logs[1],job.wrapper_type) for job in job_list]
         self.db_manager.insertMany(self.JOB_LIST_TABLE, jobs_data)
