@@ -18,16 +18,19 @@
 # along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.
 import locale
 import os
-from xml.dom.minidom import parseString
+from pathlib import Path
+from typing import Union, TYPE_CHECKING
 import subprocess
-
-
-from autosubmit.platforms.paramiko_platform import ParamikoPlatform
-from autosubmit.platforms.headers.local_header import LocalHeader
-
-from autosubmitconfigparser.config.basicconfig import BasicConfig
 from time import sleep
-from log.log import Log, AutosubmitError, AutosubmitCritical
+
+from autosubmit.platforms.headers.local_header import LocalHeader
+from autosubmit.platforms.paramiko_platform import ParamikoPlatform
+from autosubmit.platforms.wrappers.wrapper_factory import LocalWrapperFactory
+from autosubmitconfigparser.config.basicconfig import BasicConfig
+from log.log import Log, AutosubmitError
+
+if TYPE_CHECKING:
+    from autosubmitconfigparser.config.configcommon import AutosubmitConfig
 
 class LocalPlatform(ParamikoPlatform):
     """
@@ -49,8 +52,8 @@ class LocalPlatform(ParamikoPlatform):
     def get_checkAlljobs_cmd(self, jobs_id):
         pass
 
-    def __init__(self, expid, name, config):
-        ParamikoPlatform.__init__(self, expid, name, config)
+    def __init__(self, expid, name, config, auth_password = None):
+        ParamikoPlatform.__init__(self, expid, name, config, auth_password= auth_password)
         self.cancel_cmd = None
         self.mkdir_cmd = None
         self.del_cmd = None
@@ -64,8 +67,14 @@ class LocalPlatform(ParamikoPlatform):
         self.job_status['RUNNING'] = ['0']
         self.job_status['QUEUING'] = []
         self.job_status['FAILED'] = []
+        self._allow_wrappers = True
+        self._wrapper = LocalWrapperFactory(self)
+
         self.update_cmds()
 
+    def create_a_new_copy(self):
+        return LocalPlatform(self.expid, self.name, self.config)
+
     def update_cmds(self):
         """
         Updates commands for platforms
@@ -94,34 +103,55 @@ class LocalPlatform(ParamikoPlatform):
     def get_submitted_job_id(self, output, x11 = False):
         return output
 
-    def jobs_in_queue(self):
-        dom = parseString('')
-        jobs_xml = dom.getElementsByTagName("JB_job_number")
-        return [int(element.firstChild.nodeValue) for element in jobs_xml]
-
     def get_submit_cmd(self, job_script, job, hold=False, export=""):
-        wallclock = self.parse_time(job.wallclock)
-        seconds = int(wallclock.days * 86400 + wallclock.seconds * 60)
+        if job:  # Not intuitive at all, but if it is not a job, it is a wrapper
+            seconds = job.wallclock_in_seconds
+        else:
+            # TODO for another branch this, it is to add a timeout to the wrapped jobs even if the wallclock is 0, default to 2 days
+            seconds = 60*60*24*2
         if export == "none" or export == "None" or export is None or export == "":
             export = ""
         else:
             export += " ; "
-        return self.get_call(job_script, job, export=export,timeout=seconds)
+        command = self.get_call(job_script, job, export=export, timeout=seconds)
+        return f"cd {self.remote_log_dir} ; {command}"
 
     def get_checkjob_cmd(self, job_id):
         return self.get_pscall(job_id)
 
-    def connect(self, reconnect=False):
+    def connect(self, as_conf: 'AutosubmitConfig', reconnect: bool = False, log_recovery_process: bool = False) -> None:
+        """
+        Establishes an SSH connection to the host.
+
+        :param as_conf: The Autosubmit configuration object.
+        :param reconnect: Indicates whether to attempt reconnection if the initial connection fails.
+        :param log_recovery_process: Specifies if the call is made from the log retrieval process.
+        :return: None
+        """
         self.connected = True
-    def test_connection(self):
-        self.connected = True
-    def restore_connection(self):
+        if log_recovery_process:
+            self.spawn_log_retrieval_process(as_conf)
+
+    def test_connection(self,as_conf):
+        if not self.connected:
+            self.connect(as_conf)
+
+    def restore_connection(self, as_conf: 'AutosubmitConfig', log_recovery_process: bool = False) -> None:
+        """
+        Restores the SSH connection to the platform.
+
+        :param as_conf: The Autosubmit configuration object used to establish the connection.
+        :type as_conf: AutosubmitConfig
+        :param log_recovery_process: Indicates that the call is made from the log retrieval process.
+        :type log_recovery_process: bool
+        """
         self.connected = True
 
     def check_Alljobs(self, job_list, as_conf, retries=5):
         for job,prev_job_status in job_list:
             self.check_job(job)
-    def send_command(self, command,ignore_log=False, x11 = False):
+
+    def send_command(self, command, ignore_log=False, x11 = False):
         lang = locale.getlocale()[1]
         if lang is None:
             lang = locale.getdefaultlocale()[1]
@@ -138,11 +168,18 @@ class LocalPlatform(ParamikoPlatform):
 
         return True
 
-    def send_file(self, filename, check=True):
-        self.check_remote_log_dir()
-        self.delete_file(filename,del_cmd=True)
-        command = '{0} {1} {2}'.format(self.put_cmd, os.path.join(self.tmp_path, filename),
-                                       os.path.join(self.tmp_path, 'LOG_' + self.expid, filename))
+    def send_file(self, filename: str, check: bool = True) -> bool:
+        """
+        Sends a file to a specified location using a command.
+
+        :param filenames: The name of the file to send.
+        :type filenames: str
+        :param check: Unused in this platform.
+        :type check: bool
+        :return: True if the file was sent successfully.
+        :rtype: bool
+        """
+        command = f'{self.put_cmd} {os.path.join(self.tmp_path, Path(filename).name)} {os.path.join(self.tmp_path, "LOG_" + self.expid, Path(filename).name)}; chmod 770 {os.path.join(self.tmp_path, "LOG_" + self.expid, Path(filename).name)}'
         try:
             subprocess.check_call(command, shell=True)
         except subprocess.CalledProcessError:
@@ -152,6 +189,26 @@ class LocalPlatform(ParamikoPlatform):
             raise
         return True
 
+    def remove_multiple_files(self, filenames: str) -> str:
+        """
+        Creates a shell script to remove multiple files in the remote and sets the appropriate permissions.
+
+        :param filenames: A string containing the filenames to be removed.
+        :type filenames: str
+        :return: An empty string.
+        :rtype: str
+        """
+        # This function is a copy of the slurm one
+        log_dir = os.path.join(self.tmp_path, 'LOG_{0}'.format(self.expid))
+        multiple_delete_previous_run = os.path.join(
+            log_dir, "multiple_delete_previous_run.sh")
+        if os.path.exists(log_dir):
+            lang = locale.getlocale()[1]
+            if lang is None:
+                lang = 'UTF-8'
+            open(multiple_delete_previous_run, 'wb+').write(("rm -f" + filenames).encode(lang))
+            os.chmod(multiple_delete_previous_run, 0o770)
+        return ""
 
     def get_file(self, filename, must_exist=True, relative_path='',ignore_log = False,wrapper_failed=False):
         local_path = os.path.join(self.tmp_path, relative_path)
@@ -175,37 +232,30 @@ class LocalPlatform(ParamikoPlatform):
         return True
 
     # Moves .err .out
-    def check_file_exists(self, src,wrapper_failed=False):
+    def check_file_exists(self, src: str, wrapper_failed: bool = False, sleeptime: int = 1, max_retries: int = 1) -> bool:
         """
-        Moves a file on the platform
-        :param src: source name
-        :type src: str
-        :param: wrapper_failed: if True, the wrapper failed.
-        :type wrapper_failed: bool
+        Checks if a file exists in the platform.
 
+        :param src: source name.
+        :type src: str
+        :param wrapper_failed: Checks inner jobs files. Defaults to False.
+        :type wrapper_failed: bool
+        :param sleeptime: Time to sleep between retries. Defaults to 1.
+        :type sleeptime: int
+        :param max_retries: Maximum number of retries. Defaults to 1.
+        :type max_retries: int
+        :return: True if the file exists, False otherwise.
+        :rtype: bool
         """
-        file_exist = False
-        sleeptime = 5
-        remote_path = os.path.join(self.get_files_path(), src)
-        retries = 0
-        max_retries = 3
-        while not file_exist and retries < max_retries:
-            try:
-                file_exist = os.path.isfile(os.path.join(self.get_files_path(),src))
-                if not file_exist:  # File doesn't exist, retry in sleep-time
-                    Log.debug("{2} File still no exists.. waiting {0}s for a new retry ( retries left: {1})", sleeptime,
-                             max_retries - retries, remote_path)
-                    if not wrapper_failed:
-                        sleep(sleeptime)
-                        sleeptime = sleeptime + 5
-                        retries = retries + 1
-                    else:
-                        retries = 9999
-            except BaseException as e:  # Unrecoverable error
-                Log.printlog("File does not exist, logs {0} {1}".format(self.get_files_path(),src),6001)
-                file_exist = False  # won't exist
-                retries = 999  # no more retries
-        return file_exist
+        # This function has a short sleep as the files are locally
+        sleeptime = 1
+        for i in range(max_retries):
+            if os.path.isfile(os.path.join(self.get_files_path(), src)):
+                return True
+            sleep(sleeptime)
+        Log.warning("File {0} does not exist".format(src))
+        return False
+
 
     def delete_file(self, filename,del_cmd  = False):
         if del_cmd:
@@ -222,11 +272,13 @@ class LocalPlatform(ParamikoPlatform):
     def move_file(self, src, dest, must_exist=False):
         """
         Moves a file on the platform (includes .err and .out)
-        :param src: source name
+
+        :param src: source name.
         :type src: str
-        :param dest: destination name
-        :param must_exist: ignore if file exist or not
+        :param dest: destination name.
         :type dest: str
+        :param must_exist: ignore if file exist or not.
+        :type must_exist: bool
         """
         path_root = ""
         try:
@@ -234,8 +286,6 @@ class LocalPlatform(ParamikoPlatform):
             os.rename(os.path.join(path_root, src),os.path.join(path_root, dest))
             return True
         except IOError as e:
-            raise AutosubmitError('File {0} does not exists, something went wrong with the platform'.format(
-                path_root), 6004, str(e))
             if must_exist:
                 raise AutosubmitError("File {0} does not exists".format(
                     os.path.join(path_root,src)), 6004, str(e))
@@ -268,3 +318,52 @@ class LocalPlatform(ParamikoPlatform):
         :type remote_logs: (str, str)
         """
         return
+
+    def check_completed_files(self, sections: str = None) -> str:
+        """
+        Checks for completed files in the remote log directory.
+        This function is used to check inner_jobs of a wrapper.
+
+        :param sections: Space-separated string of sections to check for completed files. Defaults to None.
+        :type sections: str
+        :return: The output if the command is successful, None otherwise.
+        :rtype: str
+        """
+        # Clone of the slurm one.
+        command = "find %s " % self.remote_log_dir
+        if sections:
+            for i, section in enumerate(sections.split()):
+                command += " -name *%s_COMPLETED" % section
+                if i < len(sections.split()) - 1:
+                    command += " -o "
+        else:
+            command += " -name *_COMPLETED"
+
+        if self.send_command(command, True):
+            return self._ssh_output
+        else:
+            return None
+
+    def get_file_size(self, src: str) -> Union[int, None]:
+        """
+        Get file size in bytes
+        :param src: file path
+        """
+        try:
+            return Path(src).stat().st_size
+        except Exception:
+            Log.debug(f"Error getting file size for {src}")
+            return None
+
+    def read_file(self, src: str, max_size: int = None) -> Union[bytes, None]:
+        """
+        Read file content as bytes. If max_size is set, only the first max_size bytes are read.
+        :param src: file path
+        :param max_size: maximum size to read
+        """
+        try:
+            with open(src, "rb") as f:
+                return f.read(max_size)
+        except Exception:
+            Log.debug(f"Error reading file {src}")
+            return None
