@@ -17,9 +17,12 @@
 # You should have received a copy of the GNU General Public License
 # along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.
 
-import textwrap
 import random
 import string
+import textwrap
+
+from typing import List
+
 
 class WrapperDirector:
     """
@@ -32,10 +35,11 @@ class WrapperDirector:
 
         header = self._builder.build_header()
         job_thread = self._builder.build_job_thread()
-        if "bash" not in header[0:15]:
-            main = self._builder.build_main()
-        else:
-            nodes,main = self._builder.build_main() #What to do with nodes?
+        #if "bash" not in header[0:15]:
+
+        main = self._builder.build_main()
+        #else:
+        #    nodes,main = self._builder.build_main() #What to do with nodes?
         # change to WrapperScript object
         wrapper_script = header + job_thread + main
         wrapper_script = wrapper_script.replace("_NEWLINE_", '\\n')
@@ -49,6 +53,7 @@ class WrapperBuilder(object):
         self.job_scripts = kwargs['jobs_scripts']
         self.threads = kwargs['threads']
         self.num_procs = kwargs['num_processors']
+        self.num_procs_value = kwargs['num_processors_value']
         self.expid = kwargs['expid']
         self.jobs_resources = kwargs.get('jobs_resources', dict())
         self.allocated_nodes = kwargs.get('allocated_nodes', '')
@@ -57,8 +62,10 @@ class WrapperBuilder(object):
         self.exit_thread = ''
         if "wallclock_by_level" in list(kwargs.keys()):
             self.wallclock_by_level = kwargs['wallclock_by_level']
+
     def build_header(self):
         return textwrap.dedent(self.header_directive) + self.build_imports()
+
     def build_imports(self):
         pass
     def build_job_thread(self):
@@ -150,7 +157,6 @@ class PythonWrapperBuilder(WrapperBuilder):
 
             def run(self):
                 jobname = self.template.replace('.cmd', '')
-                #os.system("echo $(date +%s) > "+jobname+"_STAT")
                 out = str(self.template) + ".out." + str(0)
                 err = str(self.template) + ".err." + str(0)
                 print(out+"\\n")
@@ -201,7 +207,7 @@ while total_cores > 0:
         processors_per_node = int(jobs_resources['PROCESSORS_PER_NODE'])
 processors_per_node = int(jobs_resources['PROCESSORS_PER_NODE'])
 
-        """).format(self.num_procs, str(self.jobs_resources), '\n'.ljust(13))
+        """).format(self.num_procs_value, str(self.jobs_resources), '\n'.ljust(13))
 
     def build_machinefiles(self):
         machinefile_function = self.get_machinefile_function()
@@ -439,23 +445,39 @@ for i in range(len(pid_list)):
         return ''.join(padding + line for line in text.splitlines(True))
 class PythonVerticalWrapperBuilder(PythonWrapperBuilder):
 
-    def build_sequential_threads_launcher(self, jobs_list, thread, footer=True): #fastlook
+    def build_sequential_threads_launcher(self, jobs_list: List[str], thread: str, footer: bool = True) -> str:
+        """
+        Builds a part of the vertical wrapper cmd launcher script.
+        This script writes the start and finish time of each inner_job.
+
+        :param jobs_list: List of job scripts.
+        :type jobs_list: List[str]
+        :param thread: inner_job to be executed.
+        :type thread: str
+        :param footer: If True, includes the footer in the script. Defaults to True.
+        :type footer: bool
+        :return: Part of the final vertical wrapper script.
+        :rtype: str
+        """
         sequential_threads_launcher = textwrap.dedent("""
         failed_wrapper = os.path.join(os.getcwd(),wrapper_id)
         retrials = {2}
-        total_steps = 0 
-        print("JOB.ID:"+ os.getenv('SLURM_JOBID'))
+        total_steps = 0
+        try: 
+            print("JOB.ID:"+ os.getenv('SLURM_JOBID'))
+        except:
+            print("JOB.ID")
         for i in range(len({0})):
             job_retrials = retrials
             completed = False
-            while job_retrials >= 0 and not completed:
+            fail_count = 0
+            while fail_count <= job_retrials and not completed:
                 current = {1}
                 current.start()
-                os.system("echo "+str(time.time())+" > "+scripts[i][:-4]+"_STAT_"+str(job_retrials)) #Start/submit running
+                start = int(time.time())
                 current.join({3})
-                job_retrials = job_retrials - 1
                 total_steps = total_steps + 1
-        """).format(jobs_list, thread,self.retrials,str(self.wallclock_by_level),'\n'.ljust(13))
+        """).format(jobs_list, thread, self.retrials, str(self.wallclock_by_level),'\n'.ljust(13))
 
         if footer:
             sequential_threads_launcher += self._indent(textwrap.dedent("""
@@ -464,23 +486,42 @@ class PythonVerticalWrapperBuilder(PythonWrapperBuilder):
                 failed_filename = {0}[i].replace('.cmd', '_FAILED')
                 failed_path = os.path.join(os.getcwd(), failed_filename)
                 failed_wrapper = os.path.join(os.getcwd(), wrapper_id)
-                os.system("echo "+str(time.time())+" >> "+scripts[i][:-4]+"_STAT_"+str(job_retrials+1)) #Completed
+                finish = int(time.time())
+                stat_filename = {0}[i].replace(".cmd", f"_STAT_{{fail_count}}")
+                stat_path_tmp = os.path.join(os.getcwd(),f"{{stat_filename}}.tmp")
+                print(f"Completed_file:{{completed_path}}")
+                print(f"Writting:{{stat_path_tmp}}")
+                print(f"[Start:{{start}}, Finish:{{finish}}, Fail_count:{{fail_count}}]")
+                with open(f"{{stat_path_tmp}}", "w") as file:
+                    file.write(f"{{start}}\\n")
+                    file.write(f"{{finish}}\\n")
                 if os.path.exists(completed_path):
                     completed = True
                     print(datetime.now(), "The job ", current.template," has been COMPLETED")
-                    os.system("echo COMPLETED >>  " + scripts[i][:-4]+"_STAT_"+str(job_retrials+1))
                 else:
                     print(datetime.now(), "The job ", current.template," has FAILED")
-                    os.system("echo FAILED >>  " + scripts[i][:-4]+"_STAT_"+str(job_retrials+1))
                     #{1}
+                fail_count = fail_count + 1
+
             """).format(jobs_list, self.exit_thread, '\n'.ljust(13)), 8)
             sequential_threads_launcher += self._indent(textwrap.dedent("""
+            from pathlib import Path
+            fail_count = 0 
+            while fail_count <= job_retrials:
+                try:
+                    stat_filename = {0}[i].replace(".cmd", f"_STAT_{{fail_count}}")
+                    stat_path_tmp = os.path.join(os.getcwd(),f"{{stat_filename}}.tmp")
+                    Path(stat_path_tmp).replace(stat_path_tmp.replace(".tmp",""))
+                except:
+                    print(f"Couldn't write the stat file:{{stat_path_tmp}}")
+                fail_count = fail_count + 1
             if not os.path.exists(completed_path):
                 open(failed_wrapper,'wb').close()
                 open(failed_path, 'wb').close()
                 
             if os.path.exists(failed_wrapper):
                 os.remove(os.path.join(os.getcwd(),wrapper_id))
+                print("WRAPPER_FAILED")
                 wrapper_failed = os.path.join(os.getcwd(),"WRAPPER_FAILED")
                 open(wrapper_failed, 'wb').close()
                 os._exit(1)
@@ -490,29 +531,31 @@ class PythonVerticalWrapperBuilder(PythonWrapperBuilder):
     def build_job_thread(self): # fastlook
         return textwrap.dedent("""
         class JobThread(Thread):
-            def __init__ (self, template, id_run, retrials):
+            def __init__ (self, template, id_run, retrials, fail_count):
                 Thread.__init__(self)
                 self.template = template
                 self.id_run = id_run
                 self.retrials = retrials
+                self.fail_count = fail_count
 
             def run(self):
+                print("\\n")
                 jobname = self.template.replace('.cmd', '')
-                #os.system("echo $(date +%s) > "+jobname+"_STAT")
-                out = str(self.template) + ".out." + str(self.retrials)
-                err = str(self.template) + ".err." + str(self.retrials)
-                print((out+"\\n"))
-                command = "./" + str(self.template) + " " + str(self.id_run) + " " + os.getcwd()
-                print((command+"\\n"))
-                (self.status) = getstatusoutput("timeout {0} " + command + " > " + out + " 2> " + err)
-                for i in self.status:
-                    print((str(i)+"\\n"))
+                out = str(self.template) + ".out." + str(self.fail_count)
+                err = str(self.template) + ".err." + str(self.fail_count)
+                out_path = os.path.join(os.getcwd(), out)
+                err_path = os.path.join(os.getcwd(), err)
+                template_path = os.path.join(os.getcwd(), self.template)
+                command = f"chmod +x {{template_path}}; timeout {0} {{template_path}} > {{out_path}} 2> {{err_path}}"
+                print(command)
+                getstatusoutput(command)
+                
 
                 
         """).format(str(self.wallclock_by_level),'\n'.ljust(13))
     def build_main(self):
         self.exit_thread = "os._exit(1)"
-        return self.build_sequential_threads_launcher("scripts", "JobThread(scripts[i], i, job_retrials)")
+        return self.build_sequential_threads_launcher("scripts", "JobThread(scripts[i], i, retrials, fail_count)")
 class PythonHorizontalWrapperBuilder(PythonWrapperBuilder):
 
     def build_main(self):
@@ -719,7 +762,7 @@ while total_cores > 0:
         processors_per_node = int(jobs_resources['PROCESSORS_PER_NODE'])
 
 processors_per_node = int(jobs_resources['PROCESSORS_PER_NODE'])
-        """).format(self.num_procs, str(self.jobs_resources), '\n'.ljust(13))
+        """).format(self.num_procs_value, str(self.jobs_resources), '\n'.ljust(13))
 
     def build_machinefiles(self):
         machinefile_function = self.get_machinefile_function()
@@ -970,4 +1013,4 @@ class SrunVerticalHorizontalWrapperBuilder(SrunWrapperBuilder):
     def build_main(self):
         nodelist = self.build_nodes_list()
         srun_launcher = self.build_srun_launcher("scripts_list")
-        return nodelist, srun_launcher
\ No newline at end of file
+        return nodelist, srun_launcher
